==> nipper-0.10.4.tgz: nipper-0.10.4/ScreenOS/process-policy.c <==
/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#define sos_permit_permit 0
#define sos_permit_deny 1
#define sos_permit_reject 2


struct zonePolicyRuleSOS			// NetScreenOS Policy Rule Configuration
{
	int id;
	int permit;
	int disabled;
	int log;
	struct listSOS *source;
	struct listSOS *dest;
	struct listSOS *service;
	struct zonePolicyRuleSOS *next;
};


struct policySOS					// NetScreenOS Policy Configuration
{
	int global;
	char name[32];
	char fromZone[32];
	char toZone[32];
	struct zonePolicyRuleSOS *policyRules;
	struct policySOS *next;
};




// Process admin lines
void processSOSPolicy(char *line, struct nipperConfig *nipper)
{
	// Variables
	char policyName[32];
	int policyGlobal;
	int policyId;
	int policyTop;
	int policyBefore;
	char fromZone[32];
	char toZone[32];
	char tempString[nipper->maxSize];
	int position;
	struct ciscoCommand command;
	struct policySOS *policySOSPointer;
	struct zonePolicyRuleSOS *ruleSOSPointer;
	struct zonePolicyRuleSOS *ruleSOSPreviousPointer;
	struct listSOS *listSOSPointer;

	if (nipper->debugMode == true)
		printf("Policy Line: %s\n", line);

	// Init
	policyGlobal = false;
	policyTop = false;
	policyBefore = -1;
	strcpy(policyName, "");
	strcpy(fromZone, "");
	strcpy(toZone, "");
	position = 2;
	command = splitLine(line);

	// Check for Global
	if (strcmp(command.part[position], "global") == 0)
	{
		policyGlobal = true;
		position++;
	}

	// Check for ID
	if (strcmp(command.part[position], "id") == 0)
	{
		position++;
		policyId = atoi(command.part[position]);
		position++;
	}

	// Check top
	if (strcmp(command.part[position], "top") == 0)
	{
		position++;
		policyTop = true;
	}

	// Check Before
	if (strcmp(command.part[position], "before") == 0)
	{
		position++;
		policyBefore = atoi(command.part[position]);
		position++;
	}

	// Check for a Name
	if (strcmp(command.part[position], "name") == 0)
	{
		position++;
		strncpy(policyName, command.part[position], sizeof(policyName) - 1);
		position++;
	}

	if (position < command.parts)
	{
		// Check for From and to
		if (strcmp(command.part[position], "from") == 0)
		{
			position++;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(fromZone, tempString, sizeof(fromZone) - 1);
			position += 2;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(toZone, tempString, sizeof(toZone) - 1);
			position++;
		}

		// If FROM / TO
		if ((fromZone[0] != 0) || (policyGlobal == true))
		{

			// Has the policy structure been created?
			if (nipper->sos->policies == 0)
			{
				// Create...
				nipper->sos->policies = malloc(sizeof(struct policySOS));
				// Pointer...
				policySOSPointer = nipper->sos->policies;
				// Init...
				memset(policySOSPointer, 0 , sizeof(struct policySOS));
				policySOSPointer->global = false;
			}
			else
			{

				// Search for policy structure
				policySOSPointer = nipper->sos->policies;
				if (policyGlobal == true)
				{
					while ((policySOSPointer->next != 0) && (policySOSPointer->global != true))
						policySOSPointer = policySOSPointer->next;

					// If not found
					if ((policyGlobal == true) && (policySOSPointer->global != true))
					{
						// Create...
						policySOSPointer->next = malloc(sizeof(struct policySOS));
						// Pointer...
						policySOSPointer = policySOSPointer->next;
						// Init...
						memset(policySOSPointer, 0 , sizeof(struct policySOS));
						policySOSPointer->global = false;
					}
				}
				else
				{
					while ((policySOSPointer->next != 0) && ((strcmp(policySOSPointer->fromZone, fromZone) != 0) || (strcmp(policySOSPointer->toZone, toZone) != 0)))
						policySOSPointer = policySOSPointer->next;

					// If not found
					if ((strcmp(policySOSPointer->fromZone, fromZone) != 0) || (strcmp(policySOSPointer->toZone, toZone) != 0))
					{
						// Create...
						policySOSPointer->next = malloc(sizeof(struct policySOS));
						// Pointer...
						policySOSPointer = policySOSPointer->next;
						// Init...
						memset(policySOSPointer, 0 , sizeof(struct policySOS));
						policySOSPointer->global = false;
					}
				}
			}

			// Global...
			policySOSPointer->global = policyGlobal;

			// From and To...
			if (fromZone[0] != 0)
			{
				strcpy(policySOSPointer->fromZone, fromZone);
				strcpy(policySOSPointer->toZone, toZone);
			}

			// Name...
			if (policyName[0] != 0)
				strcpy(policySOSPointer->name, policyName);

			// Create policy rule
			// If no other rules exist
			if (policySOSPointer->policyRules == 0)
			{
				// Create...
				policySOSPointer->policyRules = malloc(sizeof(struct zonePolicyRuleSOS));
				// Pointers...
				ruleSOSPointer = policySOSPointer->policyRules;
			}
			// Policy at top
			else if (policyTop == true)
			{
				// Create...
				ruleSOSPointer = malloc(sizeof(struct zonePolicyRuleSOS));
				// Pointers...
				ruleSOSPointer->next = policySOSPointer->policyRules;
				policySOSPointer->policyRules = ruleSOSPointer;
			}
			// Policy Before
			else if (policyBefore != -1)
			{
				// If first rule matches
				if (policySOSPointer->policyRules->id == policyBefore)
				{
					// Create...
					ruleSOSPointer = malloc(sizeof(struct zonePolicyRuleSOS));
					// Pointers...
					ruleSOSPointer->next = policySOSPointer->policyRules;
					policySOSPointer->policyRules = ruleSOSPointer;
				}
				// Look for rule
				else
				{
					ruleSOSPreviousPointer = policySOSPointer->policyRules;
					ruleSOSPointer = ruleSOSPreviousPointer->next;
					if (ruleSOSPointer == 0)
					{
						// Create...
						ruleSOSPreviousPointer->next = malloc(sizeof(struct zonePolicyRuleSOS));
						// Pointers...
						ruleSOSPointer = ruleSOSPreviousPointer->next;
					}
					else
					{
						// Search
						while ((ruleSOSPointer->next != 0) && (ruleSOSPointer->id != policyBefore))
						{
							ruleSOSPreviousPointer = ruleSOSPointer;
							ruleSOSPointer = ruleSOSPointer->next;
						}

						// If found
						if (ruleSOSPointer->id == policyBefore)
						{
							// Create...
							ruleSOSPreviousPointer->next = malloc(sizeof(struct zonePolicyRuleSOS));
							// Pointers...
							ruleSOSPreviousPointer = ruleSOSPreviousPointer->next;
							ruleSOSPreviousPointer->next = ruleSOSPointer;
							ruleSOSPointer = ruleSOSPreviousPointer;
						}
						else
						{
							// Create...
							ruleSOSPointer->next = malloc(sizeof(struct zonePolicyRuleSOS));
							// Pointers...
							ruleSOSPointer = ruleSOSPointer->next;
						}
					}
				}
			}
			// At the end
			else
			{
				// Search...
				ruleSOSPointer = policySOSPointer->policyRules;
				while (ruleSOSPointer->next != 0)
					ruleSOSPointer = ruleSOSPointer->next;
				// Create...
				ruleSOSPointer->next = malloc(sizeof(struct zonePolicyRuleSOS));
				// Pointers...
				ruleSOSPointer = ruleSOSPointer->next;
			}

			// Init
			memset(ruleSOSPointer, 0 , sizeof(struct zonePolicyRuleSOS));
			ruleSOSPointer->disabled = false;
			ruleSOSPointer->log = false;

			// ID
			ruleSOSPointer->id = policyId;

			// Source Address
			ruleSOSPointer->source = malloc(sizeof(struct listSOS));
			memset(ruleSOSPointer->source, 0 , sizeof(struct listSOS));
			listSOSPointer = ruleSOSPointer->source;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
			position++;

			// Destination Address
			ruleSOSPointer->dest = malloc(sizeof(struct listSOS));
			memset(ruleSOSPointer->dest, 0 , sizeof(struct listSOS));
			listSOSPointer = ruleSOSPointer->dest;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
			position++;

			// Service
			ruleSOSPointer->service = malloc(sizeof(struct listSOS));
			memset(ruleSOSPointer->service, 0 , sizeof(struct listSOS));
			listSOSPointer = ruleSOSPointer->service;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
			position++;

			while (position < command.parts)
			{
				// Permit
				if (strcmp(command.part[position], "permit") == 0)
					ruleSOSPointer->permit = sos_permit_permit;

				// Deny
				else if (strcmp(command.part[position], "deny") == 0)
					ruleSOSPointer->permit = sos_permit_deny;

				// Reject
				else if (strcmp(command.part[position], "reject") == 0)
					ruleSOSPointer->permit = sos_permit_reject;

				// Log
				else if (strcmp(command.part[position], "log") == 0)
					ruleSOSPointer->log = true;

				position++;
			}
		}

		// Disable?
		else if (strcmp(command.part[position], "disable") == 0)
		{
			policySOSPointer = nipper->sos->policies;
			while (policySOSPointer != 0)
			{
				ruleSOSPointer = policySOSPointer->policyRules;
				while (ruleSOSPointer != 0)
				{
					if (ruleSOSPointer->id == policyId)
						ruleSOSPointer->disabled = true;
					ruleSOSPointer = ruleSOSPointer->next;
				}
				policySOSPointer = policySOSPointer->next;
			}
		}
	}

	// If Additional policy settings
	else
	{

		// Loop through the policy additional setting lines
		readLine(nipper->input, line, nipper->maxSize);
		command = splitLine(line);
		while ((feof(nipper->input) == 0) && (strcmp(command.part[0], "exit") != 0))
		{
			// Debug
			if (nipper->debugMode == true)
				printf("Policy Line: %s\n", line);

			// Source
			if ((strcmp(command.part[0], "set") == 0) && (strcmp(command.part[1], "src-address") == 0))
			{
				// Find policy ID
				policySOSPointer = nipper->sos->policies;
				while (policySOSPointer != 0)
				{
					ruleSOSPointer = policySOSPointer->policyRules;
					while ((ruleSOSPointer->next != 0) && (ruleSOSPointer->id != policyId))
						ruleSOSPointer = ruleSOSPointer->next;

					// Found, add new entry
					if (ruleSOSPointer->id == policyId)
					{
						// Find last entry
						listSOSPointer = ruleSOSPointer->source;
						while (listSOSPointer->next != 0)
							listSOSPointer = listSOSPointer->next;

						// Create new entry
						listSOSPointer->next = malloc(sizeof(struct listSOS));
						memset(listSOSPointer->next, 0 , sizeof(struct listSOS));
						listSOSPointer = listSOSPointer->next;
						stripQuotes(command.part[2], tempString, nipper->maxSize);
						strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
					}
					policySOSPointer = policySOSPointer->next;
				}
			}

			// Destination
			if ((strcmp(command.part[0], "set") == 0) && (strcmp(command.part[1], "dst-address") == 0))
			{
				// Find policy ID
				policySOSPointer = nipper->sos->policies;
				while (policySOSPointer != 0)
				{
					ruleSOSPointer = policySOSPointer->policyRules;
					while ((ruleSOSPointer->next != 0) && (ruleSOSPointer->id != policyId))
						ruleSOSPointer = ruleSOSPointer->next;

					// Found, add new entry
					if (ruleSOSPointer->id == policyId)
					{
						// Find last entry
						listSOSPointer = ruleSOSPointer->dest;
						while (listSOSPointer->next != 0)
							listSOSPointer = listSOSPointer->next;

						// Create new entry
						listSOSPointer->next = malloc(sizeof(struct listSOS));
						memset(listSOSPointer->next, 0 , sizeof(struct listSOS));
						listSOSPointer = listSOSPointer->next;
						stripQuotes(command.part[2], tempString, nipper->maxSize);
						strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
					}
					policySOSPointer = policySOSPointer->next;
				}
			}

			// Service
			if ((strcmp(command.part[0], "set") == 0) && (strcmp(command.part[1], "service") == 0))
			{
				// Find policy ID
				policySOSPointer = nipper->sos->policies;
				while (policySOSPointer != 0)
				{
					ruleSOSPointer = policySOSPointer->policyRules;
					while ((ruleSOSPointer->next != 0) && (ruleSOSPointer->id != policyId))
						ruleSOSPointer = ruleSOSPointer->next;

					// Found, add new entry
					if (ruleSOSPointer->id == policyId)
					{
						// Find last entry
						listSOSPointer = ruleSOSPointer->service;
						while (listSOSPointer->next != 0)
							listSOSPointer = listSOSPointer->next;

						// Create new entry
						listSOSPointer->next = malloc(sizeof(struct listSOS));
						memset(listSOSPointer->next, 0 , sizeof(struct listSOS));
						listSOSPointer = listSOSPointer->next;
						stripQuotes(command.part[2], tempString, nipper->maxSize);
						strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
					}
					policySOSPointer = policySOSPointer->next;
				}
			}

			// Get next line
			readLine(nipper->input, line, nipper->maxSize);
			command = splitLine(line);
		}
	}
}

==> nipper-0.10.5.tgz: nipper-0.10.5/ScreenOS/process-policy.c <==
/***************************************************************************
 *   nipper - The network infrastructure parser                            *
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   fizz@titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#define sos_permit_permit 0
#define sos_permit_deny 1
#define sos_permit_reject 2


struct zonePolicyRuleSOS			// NetScreenOS Policy Rule Configuration
{
	int id;
	int permit;
	int disabled;
	int log;
	struct listSOS *source;
	struct listSOS *dest;
	struct listSOS *service;
	struct zonePolicyRuleSOS *next;
};


struct policySOS					// NetScreenOS Policy Configuration
{
	int global;
	char name[32];
	char fromZone[32];
	char toZone[32];
	struct zonePolicyRuleSOS *policyRules;
	struct policySOS *next;
};




// Process admin lines
void processSOSPolicy(char *line, struct nipperConfig *nipper)
{
	// Variables
	char policyName[32];
	int policyGlobal = false;
	int policyId = 0;
	int policyTop = false;
	int policyBefore = -1;
	char fromZone[32];
	char toZone[32];
	char tempString[nipper->maxSize];
	int position = 2;
	struct ciscoCommand command;
	struct policySOS *policySOSPointer = 0;
	struct zonePolicyRuleSOS *ruleSOSPointer = 0;
	struct zonePolicyRuleSOS *ruleSOSPreviousPointer = 0;
	struct listSOS *listSOSPointer = 0;

	if (nipper->debugMode == true)
		printf("Policy Line: %s\n", line);

	// Init
	strcpy(policyName, "");
	strcpy(fromZone, "");
	strcpy(toZone, "");
	command = splitLine(line);

	// Check for Global
	if (strcmp(command.part[position], "global") == 0)
	{
		policyGlobal = true;
		position++;
	}

	// Check for ID
	if (strcmp(command.part[position], "id") == 0)
	{
		position++;
		policyId = atoi(command.part[position]);
		position++;
	}

	// Check top
	if (strcmp(command.part[position], "top") == 0)
	{
		position++;
		policyTop = true;
	}

	// Check Before
	if (strcmp(command.part[position], "before") == 0)
	{
		position++;
		policyBefore = atoi(command.part[position]);
		position++;
	}

	// Check for a Name
	if (strcmp(command.part[position], "name") == 0)
	{
		position++;
		strncpy(policyName, command.part[position], sizeof(policyName) - 1);
		position++;
	}

	if (position < command.parts)
	{
		// Check for From and to
		if (strcmp(command.part[position], "from") == 0)
		{
			position++;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(fromZone, tempString, sizeof(fromZone) - 1);
			position += 2;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(toZone, tempString, sizeof(toZone) - 1);
			position++;
		}

		// If FROM / TO
		if ((fromZone[0] != 0) || (policyGlobal == true))
		{

			// Has the policy structure been created?
			if (nipper->sos->policies == 0)
			{
				// Create...
				nipper->sos->policies = malloc(sizeof(struct policySOS));
				// Pointer...
				policySOSPointer = nipper->sos->policies;
				// Init...
				memset(policySOSPointer, 0 , sizeof(struct policySOS));
				policySOSPointer->global = false;
			}
			else
			{

				// Search for policy structure
				policySOSPointer = nipper->sos->policies;
				if (policyGlobal == true)
				{
					while ((policySOSPointer->next != 0) && (policySOSPointer->global != true))
						policySOSPointer = policySOSPointer->next;

					// If not found
					if ((policyGlobal == true) && (policySOSPointer->global != true))
					{
						// Create...
						policySOSPointer->next = malloc(sizeof(struct policySOS));
						// Pointer...
						policySOSPointer = policySOSPointer->next;
						// Init...
						memset(policySOSPointer, 0 , sizeof(struct policySOS));
						policySOSPointer->global = false;
					}
				}
				else
				{
					while ((policySOSPointer->next != 0) && ((strcmp(policySOSPointer->fromZone, fromZone) != 0) || (strcmp(policySOSPointer->toZone, toZone) != 0)))
						policySOSPointer = policySOSPointer->next;

					// If not found
					if ((strcmp(policySOSPointer->fromZone, fromZone) != 0) || (strcmp(policySOSPointer->toZone, toZone) != 0))
					{
						// Create...
						policySOSPointer->next = malloc(sizeof(struct policySOS));
						// Pointer...
						policySOSPointer = policySOSPointer->next;
						// Init...
						memset(policySOSPointer, 0 , sizeof(struct policySOS));
						policySOSPointer->global = false;
					}
				}
			}

			// Global...
			policySOSPointer->global = policyGlobal;

			// From and To...
			if (fromZone[0] != 0)
			{
				strcpy(policySOSPointer->fromZone, fromZone);
				strcpy(policySOSPointer->toZone, toZone);
			}

			// Name...
			if (policyName[0] != 0)
				strcpy(policySOSPointer->name, policyName);

			// Create policy rule
			// If no other rules exist
			if (policySOSPointer->policyRules == 0)
			{
				// Create...
				policySOSPointer->policyRules = malloc(sizeof(struct zonePolicyRuleSOS));
				// Pointers...
				ruleSOSPointer = policySOSPointer->policyRules;
			}
			// Policy at top
			else if (policyTop == true)
			{
				// Create...
				ruleSOSPointer = malloc(sizeof(struct zonePolicyRuleSOS));
				// Pointers...
				ruleSOSPointer->next = policySOSPointer->policyRules;
				policySOSPointer->policyRules = ruleSOSPointer;
			}
			// Policy Before
			else if (policyBefore != -1)
			{
				// If first rule matches
				if (policySOSPointer->policyRules->id == policyBefore)
				{
					// Create...
					ruleSOSPointer = malloc(sizeof(struct zonePolicyRuleSOS));
					// Pointers...
					ruleSOSPointer->next = policySOSPointer->policyRules;
					policySOSPointer->policyRules = ruleSOSPointer;
				}
				// Look for rule
				else
				{
					ruleSOSPreviousPointer = policySOSPointer->policyRules;
					ruleSOSPointer = ruleSOSPreviousPointer->next;
					if (ruleSOSPointer == 0)
					{
						// Create...
						ruleSOSPreviousPointer->next = malloc(sizeof(struct zonePolicyRuleSOS));
						// Pointers...
						ruleSOSPointer = ruleSOSPreviousPointer->next;
					}
					else
					{
						// Search
						while ((ruleSOSPointer->next != 0) && (ruleSOSPointer->id != policyBefore))
						{
							ruleSOSPreviousPointer = ruleSOSPointer;
							ruleSOSPointer = ruleSOSPointer->next;
						}

						// If found
						if (ruleSOSPointer->id == policyBefore)
						{
							// Create...
							ruleSOSPreviousPointer->next = malloc(sizeof(struct zonePolicyRuleSOS));
							// Pointers...
							ruleSOSPreviousPointer = ruleSOSPreviousPointer->next;
							ruleSOSPreviousPointer->next = ruleSOSPointer;
							ruleSOSPointer = ruleSOSPreviousPointer;
						}
						else
						{
							// Create...
							ruleSOSPointer->next = malloc(sizeof(struct zonePolicyRuleSOS));
							// Pointers...
							ruleSOSPointer = ruleSOSPointer->next;
						}
					}
				}
			}
			// At the end
			else
			{
				// Search...
				ruleSOSPointer = policySOSPointer->policyRules;
				while (ruleSOSPointer->next != 0)
					ruleSOSPointer = ruleSOSPointer->next;
				// Create...
				ruleSOSPointer->next = malloc(sizeof(struct zonePolicyRuleSOS));
				// Pointers...
				ruleSOSPointer = ruleSOSPointer->next;
			}

			// Init
			memset(ruleSOSPointer, 0 , sizeof(struct zonePolicyRuleSOS));
			ruleSOSPointer->disabled = false;
			ruleSOSPointer->log = false;

			// ID
			ruleSOSPointer->id = policyId;

			// Source Address
			ruleSOSPointer->source = malloc(sizeof(struct listSOS));
			memset(ruleSOSPointer->source, 0 , sizeof(struct listSOS));
			listSOSPointer = ruleSOSPointer->source;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
			position++;

			// Destination Address
			ruleSOSPointer->dest = malloc(sizeof(struct listSOS));
			memset(ruleSOSPointer->dest, 0 , sizeof(struct listSOS));
			listSOSPointer = ruleSOSPointer->dest;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
			position++;

			// Service
			ruleSOSPointer->service = malloc(sizeof(struct listSOS));
			memset(ruleSOSPointer->service, 0 , sizeof(struct listSOS));
			listSOSPointer = ruleSOSPointer->service;
			stripQuotes(command.part[position], tempString, nipper->maxSize);
			strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
			position++;

			while (position < command.parts)
			{
				// Permit
				if (strcmp(command.part[position], "permit") == 0)
					ruleSOSPointer->permit = sos_permit_permit;

				// Deny
				else if (strcmp(command.part[position], "deny") == 0)
					ruleSOSPointer->permit = sos_permit_deny;

				// Reject
				else if (strcmp(command.part[position], "reject") == 0)
					ruleSOSPointer->permit = sos_permit_reject;

				// Log
				else if (strcmp(command.part[position], "log") == 0)
					ruleSOSPointer->log = true;

				position++;
			}
		}

		// Disable?
		else if (strcmp(command.part[position], "disable") == 0)
		{
			policySOSPointer = nipper->sos->policies;
			while (policySOSPointer != 0)
			{
				ruleSOSPointer = policySOSPointer->policyRules;
				while (ruleSOSPointer != 0)
				{
					if (ruleSOSPointer->id == policyId)
						ruleSOSPointer->disabled = true;
					ruleSOSPointer = ruleSOSPointer->next;
				}
				policySOSPointer = policySOSPointer->next;
			}
		}
	}

	// If Additional policy settings
	else
	{

		// Loop through the policy additional setting lines
		readLine(nipper->input, line, nipper->maxSize);
		command = splitLine(line);
		while ((feof(nipper->input) == 0) && (strcmp(command.part[0], "exit") != 0))
		{
			// Debug
			if (nipper->debugMode == true)
				printf("Policy Line: %s\n", line);

			// Source
			if ((strcmp(command.part[0], "set") == 0) && (strcmp(command.part[1], "src-address") == 0))
			{
				// Find policy ID
				policySOSPointer = nipper->sos->policies;
				while (policySOSPointer != 0)
				{
					ruleSOSPointer = policySOSPointer->policyRules;
					while ((ruleSOSPointer->next != 0) && (ruleSOSPointer->id != policyId))
						ruleSOSPointer = ruleSOSPointer->next;

					// Found, add new entry
					if (ruleSOSPointer->id == policyId)
					{
						// Find last entry
						listSOSPointer = ruleSOSPointer->source;
						while (listSOSPointer->next != 0)
							listSOSPointer = listSOSPointer->next;

						// Create new entry
						listSOSPointer->next = malloc(sizeof(struct listSOS));
						memset(listSOSPointer->next, 0 , sizeof(struct listSOS));
						listSOSPointer = listSOSPointer->next;
						stripQuotes(command.part[2], tempString, nipper->maxSize);
						strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
					}
					policySOSPointer = policySOSPointer->next;
				}
			}

			// Destination
			if ((strcmp(command.part[0], "set") == 0) && (strcmp(command.part[1], "dst-address") == 0))
			{
				// Find policy ID
				policySOSPointer = nipper->sos->policies;
				while (policySOSPointer != 0)
				{
					ruleSOSPointer = policySOSPointer->policyRules;
					while ((ruleSOSPointer->next != 0) && (ruleSOSPointer->id != policyId))
						ruleSOSPointer = ruleSOSPointer->next;

					// Found, add new entry
					if (ruleSOSPointer->id == policyId)
					{
						// Find last entry
						listSOSPointer = ruleSOSPointer->dest;
						while (listSOSPointer->next != 0)
							listSOSPointer = listSOSPointer->next;

						// Create new entry
						listSOSPointer->next = malloc(sizeof(struct listSOS));
						memset(listSOSPointer->next, 0 , sizeof(struct listSOS));
						listSOSPointer = listSOSPointer->next;
						stripQuotes(command.part[2], tempString, nipper->maxSize);
						strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
					}
					policySOSPointer = policySOSPointer->next;
				}
			}

			// Service
			if ((strcmp(command.part[0], "set") == 0) && (strcmp(command.part[1], "service") == 0))
			{
				// Find policy ID
				policySOSPointer = nipper->sos->policies;
				while (policySOSPointer != 0)
				{
					ruleSOSPointer = policySOSPointer->policyRules;
					while ((ruleSOSPointer->next != 0) && (ruleSOSPointer->id != policyId))
						ruleSOSPointer = ruleSOSPointer->next;

					// Found, add new entry
					if (ruleSOSPointer->id == policyId)
					{
						// Find last entry
						listSOSPointer = ruleSOSPointer->service;
						while (listSOSPointer->next != 0)
							listSOSPointer = listSOSPointer->next;

						// Create new entry
						listSOSPointer->next = malloc(sizeof(struct listSOS));
						memset(listSOSPointer->next, 0 , sizeof(struct listSOS));
						listSOSPointer = listSOSPointer->next;
						stripQuotes(command.part[2], tempString, nipper->maxSize);
						strncpy(listSOSPointer->name, tempString, sizeof(listSOSPointer->name) - 1);
					}
					policySOSPointer = policySOSPointer->next;
				}
			}

			// Get next line
			readLine(nipper->input, line, nipper->maxSize);
			command = splitLine(line);
		}
	}
}
